package cypher;

import java.io.File;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;


public class LaunchEncryptFile {
    public LaunchEncryptFile() {
        super();
    }

    List arguments;
    boolean argsError = false;
    List listErrorArgs = new ArrayList();
    HashSet usedFlags = new HashSet();
    Map options = new HashMap();

    String fileToEncrypt;
    //String URI;
    String keyStoreFileName;
    String keyStorePassword;
    String keyStoreAlias;
    String outputFile;
    //String outputPath;

    EncryptFile encryptFile;

    private static Logger LOGGER = Logger.getLogger("cypher.LaunchEncryptFile");

    /**
     * Executable method.
     * @param args: Args must be a combination of parameter specification (flag, -something) and value (without -) see 
     * initUsedArgs for a list of parameters specification:
     * (-k keystoreFile -p keystorePassword -os keystoreAlias -fte FileNameToEncrypt -f OutputFileName)
     */
    public static void main(String[] args) {
        LaunchEncryptFile esf = new LaunchEncryptFile();
        if (esf.loadArgs(args)) {
            esf.launch();
        }
    }

    /**
     * Create a DecryptFile object and launch it (via constructor)
     */
    public void launch() {
        try {
                encryptFile = new EncryptFile();
                encryptFile.encrypt(keyStoreFileName, keyStoreAlias, keyStorePassword, fileToEncrypt, outputFile);
        } catch (Exception e) {
            LOGGER.warning("Exception " + e.getMessage());
        }
    }

    /**
     * Load and initilize arguments.
     * @param args: Arguments, same as in main.
     * @return: true if everything is ok. false if there is any problem. (note, true/false as boolean primitive, not Boolean object).
     */
    public boolean loadArgs(String[] args) {
        initUsedArgs();
        boolean resFlag = false;
        parse(args);
        String resStr = checkArgs();
        if (resStr.length() == 0) {
            resFlag = true;
        } else {
            LOGGER.warning("Syntax error. Use with params:\n" +
                    resStr);
            resFlag = false;
        }
        return resFlag;
    }

    /**
     * Initializes usedFlag with a list of allowed flags (parameter specifications)
     */
    public void initUsedArgs() {
        usedFlags.add("-k"); //Java KeyStore file
        usedFlags.add("-p"); //Java KeyStore password
        usedFlags.add("-os"); //Java KeyStore alias
        //usedFlags.add("-d"); //FilePath of data instances. Used as URI
        usedFlags.add("-fte"); //File/URL to sign. Full file path
        //usedFlags.add("-o"); //Output directory?
        usedFlags.add("-f"); //Output File/URL? (why different from output directory?)
    }

    /**
     *Do whatever checking on the arguments
     * @return: String with the error/s (if any)
     */
    public String checkArgs() {
        String strRes = "";
        String param, message;
        if (options == null || options.isEmpty()) {
            message =
                    "Use with parameters: " + "-k keystoreFile " + "-p keystorePassword " + "-os keystoreAlias " + "-fte FileNameToEncrypt " +
                    "-f OutputFileName";
            strRes = strRes + message;
        } else {
            param = "-k";
            message = "Missing parameter -k. Use: -k keystoreFile\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                keyStoreFileName = ((String)options.get(param)).trim();
            }
            param = "-p";
            message = "Missing parameter -p. Use: -p keystorePassword\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                keyStorePassword = ((String)options.get(param)).trim();
            }
            param = "-os";
            message = "Missing parameter -os. Use: -os keystoreAlias\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                keyStoreAlias = ((String)options.get(param)).trim();
            }
            param = "-fte";
            message = "Missing parameter -fte. Use: -fte FileNameToEncrypt\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                fileToEncrypt = ((String)options.get(param)).trim();
                try {
                    File f = new File(fileToEncrypt);
                    if ((f.toString()).trim().length() <= 0) {
                        strRes = strRes + message;
                    }
                } catch (Exception ex) {
                    strRes = strRes + message;
                    fileToEncrypt = "";
                }
            }
            param = "-f";
            message = "Missing or bad parameter -f. Use: \"-f OutputFileName\".\n";
            if (!options.containsKey(param) || ((String)options.get(param)).trim().length() == 0) {
                strRes = strRes + message;
            } else {
                outputFile = ((String)options.get(param)).trim();
                try {
                    File f = new File(outputFile);
                } catch (Exception ex) {
                    strRes = strRes + message;
                    outputFile = "";
                }
            }
        }

        return strRes;
    }


    /**
     * Parse parameters from arguments to its corresponding place (inside options)
     * @param args: Same as in the main method
     * @return: true if everything is ok. false otherwise.
     */
    public boolean parse(String[] args) {
        for (int n = 0; n < args.length; ++n) {
            if (args[n].charAt(0) == '-') {
                String name = args[n];
                String value = null;

                if (usedFlags.contains(args[n]) && n < args.length - 1){
                    value = args[++n];
                    options.put(name, value);
                }
            }

            else {
                listErrorArgs.add(args[n]);
                argsError = true;
            }
        }
        return argsError;
    }
}
